@@ -704,11 +704,33 @@ def generate_orthogonal_vector(v: object) -> pd.DataFrame:
     return w
 
 
-def generate_vectors_bouquet(vpivot, bouquet_size: int, min_corr: float,
+def generate_vectors_bouquet(vpivot: Union[np.ndarray, Tuple[float], pd.DataFrame],
+                             bouquet_size: int,
+                             min_corr: float = 0.0,
                              positive: bool = True,
                              col_shift: int = 0,
                              row_shift: int = 0) -> pd.DataFrame:
-
+    """
+    generate_vectors_bouquet:
+    generates a set of vectors around a given pivot vector, having a minimum prescribed mutual correlation
+    Parameters
+    ----------
+    vpivot: Union[np.ndarray, Tuple[float], pd.DataFrame]
+        the pivot vector used to create the bouquet
+    bouquet_size: int
+        number of vectors in the bouquet
+    min_corr: float = 0.0
+        minimum correlation between two vectors
+    positive: bool = True
+        if true, all vectors have positive coordinates
+    col_shift: int = 0
+        the integer shift applied to the column numbering
+    row_shift: int = 0
+        the integer shift applied to the line numbering
+    Returns
+    -------
+    A dataframe having one line per space coordinate (common dimension), one column per vector in the bouquet
+    """
     vpivot = pd.DataFrame(vpivot)
     nr, _ = vpivot.shape
     df_bouquet = pd.DataFrame(index=range(row_shift, row_shift+nr),
@@ -733,10 +755,29 @@ def generate_vectors_bouquet(vpivot, bouquet_size: int, min_corr: float,
     return df_bouquet
 
 
-def generate_clusters_centroids(ndim: int, max_corr: float) -> pd.DataFrame:
+def generate_clusters_centroids(ndim: int, max_corr: float, n_centroids: int = 0) -> pd.DataFrame:
+    """
+    Generates a vector set of fixed dimension having pairwise bounded correlation.
+    By default, the set size is the same as the vector space dimension.
+    TODO: handle the case where the set size is greater thar the dimension, this may result in a contradiction
+    TODO: if the correlation bound is too small or null
+    Parameters
+    ----------
+    ndim: int
+        size of vectors
+    max_corr: float
+        maximum pairwise correlation
+    n_centroids
+        number of generated vectors
 
+    Returns
+    -------
+
+    """
     assert (ndim >= 1)
     assert (0 <= max_corr < 1)
+    if n_centroids == 0:
+        n_centroids = ndim
     a = ndim / 4.0 * (1 - max_corr / 3)
     b = 1 - max_corr
     c = - max_corr
@@ -783,7 +824,30 @@ def generate_integer_partition(ntot: int, nbpart: int, nmin: int = 1):
 def generate_clusterized_vectors(vsize: int, nbvect: int, nclusters: int,
                                  clusterminsize: int, max_inter_corr: float,
                                  min_intra_corr: float, random_norms: bool = True) -> pd.DataFrame:
+    """
+    generate_clusterized_vectors: generates several bouquets or set of vectors grouped by correlation clusters.
 
+    Parameters
+    ----------
+    vsize: int
+        common vector length - the dimension of the surrounding vector space
+    nbvect: int
+        total number of vector to generate
+    nclusters: int
+        number of vector clusters to generate
+    clusterminsize: int
+        minimum size of a cluster
+    max_inter_corr: float
+        maximum correlation between the centroids of two distinct clusters
+    min_intra_corr: float
+        minimum correlation between two vectors belonging to the same cluster
+    random_norms: bool
+        if true, the norms of generated vectors are log-normally distributed
+
+    Returns
+    -------
+    A dataframe having 'vsize' lines and 'nbvects' columns. The columns are grouped into the desired 'nclusters' groups.
+    """
     clustersizes = generate_integer_partition(ntot=nbvect, nbpart=nclusters, nmin=clusterminsize)
     centroids = generate_clusters_centroids(ndim=vsize, max_corr=max_inter_corr)
     df_res = pd.DataFrame(index=range(vsize))
@@ -792,14 +856,16 @@ def generate_clusterized_vectors(vsize: int, nbvect: int, nclusters: int,
     row_shift = 0
     col_shift = 0
     for icluster, csize in enumerate(clustersizes):
-        centroid = centroids.iloc[:, icluster]
-        df_bouquet = generate_vectors_bouquet(vpivot=centroid,
-                                              bouquet_size=int(csize),
-                                              min_corr=min_intra_corr,
-                                              col_shift=col_shift
-                                              )
-        df_bouquet *= 1 + factors[icluster]
-        df_res = pd.concat([df_res, df_bouquet], axis=1)
+        if icluster < len(centroids.columns):
+            centroid = centroids.iloc[:, icluster]
+            df_bouquet = generate_vectors_bouquet(vpivot=centroid,
+                                                  bouquet_size=int(csize),
+                                                  min_corr=min_intra_corr,
+                                                  col_shift=col_shift
+                                                  )
+            df_bouquet *= 1 + factors[icluster]
+
+            df_res = pd.concat([df_res, df_bouquet], axis=1)
         col_shift += int(csize)
     return df_res
 
