@@ -81,10 +81,9 @@ if __name__ == "__main__":
 
         df_scree_plot = nmf.generate_scree_plot(generated_M, ncmin=ncmin, ncmax=ncmax, known_ncomp=nb_clusters,
                                                 do_entropies=do_ent)
-        df_mini_scree_plots = nmf.add_miniscree_plots_from_scree_plot(df_examples=df_mini_scree_plots,
-                                                                      scree_plot=df_scree_plot,
-                                                                      known_ncomp=nb_clusters,
-                                                                      ncomp_margin=1)
+        df_mini_scree_plots = nmf.collect_windows_from_scree_plot(df_collected_windows=df_mini_scree_plots,
+                                                                  df_scree_plot=df_scree_plot, known_ncomp=nb_clusters,
+                                                                  ncomp_margin=1)
 
         if itrial % writefreq == 0 or itrial == nruns-1:
             print(f"Saving iteration {itrial} in {outputpath}")
